package com.atguigu.order.service.impl;

import com.alibaba.nacos.client.naming.utils.CollectionUtils;
import com.atguigu.constant.MessageConstant;
import com.atguigu.context.BaseContext;
import com.atguigu.exception.BunnyException;
import com.atguigu.feign.cart.CartFeignClient;
import com.atguigu.feign.product.ProductFeignClient;
import com.atguigu.feign.user.UserFeignClient;
import com.atguigu.order.mapper.OrderInfoMapper;
import com.atguigu.order.mapper.OrderItemMapper;
import com.atguigu.order.mapper.OrderLogMapper;
import com.atguigu.order.service.OrderInfoService;
import com.atguigu.order.service.module.OrderInfoServiceModule;
import com.atguigu.spzx.model.dto.h5.OrderInfoDto;
import com.atguigu.spzx.model.entity.order.OrderInfo;
import com.atguigu.spzx.model.entity.order.OrderItem;
import com.atguigu.spzx.model.entity.order.OrderLog;
import com.atguigu.spzx.model.entity.product.ProductSku;
import com.atguigu.spzx.model.entity.user.UserAddress;
import com.atguigu.spzx.model.entity.user.UserInfo;
import com.atguigu.spzx.model.vo.h5.TradeVo;
import com.atguigu.spzx.model.vo.result.ResultCodeEnum;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
@Slf4j
public class OrderInfoServiceImpl implements OrderInfoService {
    @Autowired
    private OrderInfoServiceModule orderInfoServiceModule;
    @Autowired
    private CartFeignClient cartFeignClient;
    @Autowired
    private ProductFeignClient productFeignClient;
    @Autowired
    private UserFeignClient userFeignClient;
    @Autowired
    private OrderInfoMapper orderInfoMapper;
    @Autowired
    private OrderItemMapper orderItemMapper;
    @Autowired
    private OrderLogMapper orderLogMapper;

    /**
     * 确认下单
     *
     * @return 结算实体类
     */
    @Override
    public TradeVo getTrade() {
        // 远程调用获取购物车中商品列表
        ArrayList<OrderItem> orderItems = orderInfoServiceModule.getOrderItems();
        // 计算总金额
        BigDecimal totalAmount;
        totalAmount = orderItems.stream()
                .map(orderItem -> orderItem.getSkuPrice().multiply(new BigDecimal(orderItem.getSkuNum())))
                .reduce(BigDecimal.ZERO, BigDecimal::add);

        TradeVo tradeVo = new TradeVo();
        tradeVo.setOrderItemList(orderItems);
        tradeVo.setTotalAmount(totalAmount);

        return tradeVo;
    }

    /**
     * 提交订单
     *
     * @param orderInfoDto OrderInfoDto
     * @return Long
     */
    @Override
    public Long submitOrder(OrderInfoDto orderInfoDto) {
        // 数据校验
        List<OrderItem> orderItemList = orderInfoDto.getOrderItemList();
        if (CollectionUtils.isEmpty(orderItemList)) {
            log.error("com.atguigu.order.service.impl===>submitOrder,orderItemList为空");
            throw new BunnyException(ResultCodeEnum.DATA_ERROR);
        }
        // 检验商品库存是否充足，遍历list集合
        orderItemList.forEach(orderItem -> {
            ProductSku productSku = productFeignClient.getBySkuId(orderItem.getSkuId()).getData();
            if (productSku == null) {
                log.error("com.atguigu.order.service.impl===>submitOrder,productSku");
                throw new BunnyException(MessageConstant.DATA_IS_EMPTY);
            }
            // 检验每个库存是否充足
            if (productSku.getStockNum() < orderItem.getSkuNum()) {
                log.error("com.atguigu.order.service.impl===>submitOrder,库存不足");
                throw new BunnyException(MessageConstant.STOCK_LESS);
            }
        });

        // 构建订单数据，保存订单
        UserInfo userInfo = BaseContext.getUserInfo();
        OrderInfo orderInfo = new OrderInfo();
        // 订单编号
        orderInfo.setOrderNo(String.valueOf(System.currentTimeMillis()));
        // 用户id
        orderInfo.setUserId(userInfo.getId());
        // 用户昵称
        orderInfo.setNickName(userInfo.getNickName());
        // 用户收货地址信息
        UserAddress userAddress = userFeignClient.getUserAddress(orderInfoDto.getUserAddressId()).getData();
        orderInfo.setReceiverName(userAddress.getName());
        orderInfo.setReceiverPhone(userAddress.getPhone());
        orderInfo.setReceiverTagName(userAddress.getTagName());
        orderInfo.setReceiverProvince(userAddress.getProvinceCode());
        orderInfo.setReceiverCity(userAddress.getCityCode());
        orderInfo.setReceiverDistrict(userAddress.getDistrictCode());
        orderInfo.setReceiverAddress(userAddress.getFullAddress());
        // 订单金额
        BigDecimal totalAmount = new BigDecimal(0);
        for (OrderItem orderItem : orderItemList) {
            totalAmount = totalAmount.add(orderItem.getSkuPrice().multiply(new BigDecimal(orderItem.getSkuNum())));
        }
        orderInfo.setTotalAmount(totalAmount);
        orderInfo.setCouponAmount(new BigDecimal(0));
        orderInfo.setOriginalTotalAmount(totalAmount);
        orderInfo.setFeightFee(orderInfoDto.getFeightFee());
        orderInfo.setPayType(2);
        orderInfo.setOrderStatus(0);
        orderInfoMapper.save(orderInfo);

        // 保存订单明细
        for (OrderItem orderItem : orderItemList) {
            orderItem.setOrderId(orderInfo.getId());
            orderItemMapper.save(orderItem);
        }

        // 记录日志
        OrderLog orderLog = new OrderLog();
        orderLog.setOrderId(orderInfo.getId());
        orderLog.setProcessStatus(0);
        orderLog.setNote("提交订单");
        orderLogMapper.save(orderLog);

        // 5、清空购物车数据
        cartFeignClient.deleteChecked();
        // 6、返回订单id
        return orderInfo.getId();
    }

    /**
     * 获取订单信息
     *
     * @param orderId 订单Id
     * @return OrderInfo
     */
    @Override
    public OrderInfo getOrderInfo(Long orderId) {
        log.info("com.atguigu.order.service.impl===>getOrderInfo");
        return orderInfoMapper.getById(orderId);
    }

    /**
     * 立即购买
     *
     * @param skuId 商品Id
     * @return 结算实体类
     */
    @Override
    public TradeVo buy(Long skuId) {
        // 查询商品
        ProductSku productSku = productFeignClient.getBySkuId(skuId).getData();
        List<OrderItem> orderItemList = new ArrayList<>();
        OrderItem orderItem = new OrderItem();
        orderItem.setSkuId(skuId);
        orderItem.setSkuName(productSku.getSkuName());
        orderItem.setSkuNum(1);
        orderItem.setSkuPrice(productSku.getSalePrice());
        orderItem.setThumbImg(productSku.getThumbImg());
        orderItemList.add(orderItem);

        // 计算总金额
        BigDecimal totalAmount = productSku.getSalePrice();
        TradeVo tradeVo = new TradeVo();
        tradeVo.setTotalAmount(totalAmount);
        tradeVo.setOrderItemList(orderItemList);

        // 返回
        return tradeVo;
    }

    /**
     * 获取订单分页列表
     *
     * @param page        page
     * @param limit       limit
     * @param orderStatus orderStatus
     * @return PageInfo<OrderInfo>
     */
    @Override
    public PageInfo<OrderInfo> findUserPage(Integer page, Integer limit, Integer orderStatus) {
        Long userId = BaseContext.getUserInfo().getId();
        List<OrderInfo> orderInfoList = orderInfoMapper.findUserPage(userId, orderStatus);

        orderInfoList.forEach(orderInfo -> {
            List<OrderItem> orderItem = orderItemMapper.findByOrderId(orderInfo.getId());
            orderInfo.setOrderItemList(orderItem);
        });

        return new PageInfo<>(orderInfoList);
    }

    /**
     * 获取订单信息
     *
     * @param orderNo 订单好
     * @return OrderInfo
     */
    @Override
    public OrderInfo getByOrderNo(String orderNo) {
        OrderInfo orderInfo = orderInfoMapper.getOrderInfoByOrderNo(orderNo);
        List<OrderItem> orderItemList = orderItemMapper.findByOrderId(orderInfo.getId());
        orderInfo.setOrderItemList(orderItemList);
        return orderInfo;
    }

    /**
     * 获取订单分页列表
     *
     * @param orderNo     orderNo
     * @param orderStatus orderStatus
     */
    @Override
    public void updateOrderStatus(String orderNo, Integer orderStatus) {
        // 更新订单状态
        OrderInfo orderInfo = orderInfoMapper.getByOrderNo(orderNo);
        orderInfo.setOrderStatus(1);
        orderInfo.setPayType(orderStatus);
        orderInfo.setPaymentTime(new Date());
        orderInfoMapper.updateById(orderInfo);

        // 记录日志
        OrderLog orderLog = new OrderLog();
        orderLog.setOrderId(orderInfo.getId());
        orderLog.setProcessStatus(1);
        orderLog.setNote("支付宝支付成功");
        orderLogMapper.save(orderLog);
    }
}
