package com.atguigu.user.service.impl;

import com.atguigu.constant.MessageConstant;
import com.atguigu.constant.UserConstant;
import com.atguigu.context.BaseContext;
import com.atguigu.exception.BunnyException;
import com.atguigu.spzx.model.dto.h5.UserLoginDto;
import com.atguigu.spzx.model.dto.h5.UserRegisterDto;
import com.atguigu.spzx.model.entity.user.UserInfo;
import com.atguigu.spzx.model.vo.h5.UserInfoVo;
import com.atguigu.user.mapper.UserInfoMapper;
import com.atguigu.user.service.UserInfoService;
import com.atguigu.utils.EmptyUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;
import org.springframework.util.StringUtils;

import java.util.Objects;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

@Service
public class UserInfoServiceImpl implements UserInfoService {
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;
    @Autowired
    private UserInfoMapper userInfoMapper;
    @Autowired
    private EmptyUtil emptyUtil;

    /**
     * 会员注册
     *
     * @param userRegisterDto 注册对象
     */
    @Override
    public void register(UserRegisterDto userRegisterDto) {
        emptyUtil.isEmpty(userRegisterDto.getUsername(), MessageConstant.SAVE_DTO_IS_NULL);
        // 获取数据
        String username = userRegisterDto.getUsername();
        String password = userRegisterDto.getPassword();
        String code = userRegisterDto.getCode();

        // 从Redis中获取发送的验证码
        Object redisCode = redisTemplate.opsForValue().get(username);
        emptyUtil.isEmpty(redisCode, MessageConstant.VERIFICATION_CODE_IS_EMPTY);
        if (!Objects.equals(String.valueOf(redisCode), code)) {
            throw new BunnyException(MessageConstant.VERIFICATION_CODE_ERROR);
        }

        // 校验用户名不能相同
        UserInfo userInfo = userInfoMapper.selectByUsername(username);
        emptyUtil.isNotEmpty(userInfo, MessageConstant.USER_DOES_IS_EXIST);

        userInfo = new UserInfo();
        BeanUtils.copyProperties(userRegisterDto, userInfo);
        userInfo.setPassword(DigestUtils.md5DigestAsHex(password.getBytes()));
        userInfo.setPhone(username);
        userInfo.setStatus(1);
        userInfo.setStatus(0);
        userInfo.setAvatar(UserConstant.USER_AVATAR);

        // 保存用户信息
        userInfoMapper.save(userInfo);

        // 删除Redis中的数据
        redisTemplate.delete(username);
    }

    /**
     * 会员登录
     *
     * @param userLoginDto 用户登录请求参数
     * @return 返回token
     */
    @Override
    public String login(UserLoginDto userLoginDto) {
        String password = userLoginDto.getPassword();
        String username = userLoginDto.getUsername();
        if (!StringUtils.hasText(username) || !StringUtils.hasText(password)) {
            throw new BunnyException(MessageConstant.LOGIN_DTO_IS_EMPTY);
        }

        // 判断用户是否存在
        UserInfo userInfo = userInfoMapper.selectByUsername(username);
        emptyUtil.isEmpty(userInfo, MessageConstant.USER_DOES_NOT_EXIST);

        // 判断密码是否相同
        String encryptPassword = DigestUtils.md5DigestAsHex(password.getBytes());
        if (!userInfo.getPassword().equals(encryptPassword)) {
            throw new BunnyException(MessageConstant.PASSWORD_ERROR);
        }
        // 存入用户信息到Redis
        String token = UUID.randomUUID().toString();
        redisTemplate.opsForValue().set(token, userInfo, 30, TimeUnit.DAYS);
        return token;
    }

    /**
     * 获取当前登录用户信息
     *
     * @param token token
     * @return 用户信息
     */
    @Override
    public UserInfoVo getCurrentUserInfo(String token) {
        UserInfo userInfo = BaseContext.getUserInfo();
        UserInfoVo userInfoVo = new UserInfoVo();
        BeanUtils.copyProperties(userInfo, userInfoVo);

        return userInfoVo;
    }
}
